#include "camera.h"
#include "rayzor.h"

Camera::Camera()
{
	type = NODE_CAMERA;
	fov = M_PI / 4.0;
	set_position(Vector3(0, 0, 10));
}

void Camera::set_target(const Vector3 &target)
{
	this->target.set_position(target);
}

Vector3 Camera::get_target() const
{
	return target.get_position();
}

void Camera::set_fov(float fov)
{
	this->fov = fov;
}

float Camera::get_fov() const
{
	return fov;
}

void Camera::calc_matrix() const
{
	if(!xform_valid) {
		xform.set_identity();
		xform.lookat(pos, target.get_position(), Vector3(0, 1, 0));
		xform_valid = true;
	}
}

void Camera::draw(bool emph) const
{
}

bool Camera::intersect(const Ray &ray, RayHit *hit) const
{
	return false;
}

Ray Camera::get_primary_ray(int x, int y) const
{
	calc_inv_matrix();

	float aspect = (float)fb_width / (float)fb_height;
	float pwidth = 2.0 * aspect / (float)fb_width;
	float pheight = 2.0 / (float)fb_height;

	Vector3 dir;
	dir.x = (float)x * pwidth - aspect;
	dir.y = 1.0 - (float)y * pheight;
	dir.z = -1.0 / tan(fov / 2.0);
	dir.normalize();

	return transform(inv_xform, Ray(Vector3(0, 0, 0), dir));
}
