#include "light.h"

Light::Light()
{
	type = NODE_LIGHT;
	intens = 1.0;
	color.x = color.y = color.z = 1.0;
	atten.x = 1.0;
	atten.y = 0.0;
	atten.z = 0.0;
}

void Light::set_intensity(float val)
{
	intens = val;
}

float Light::get_intensity() const
{
	return intens;
}

void Light::set_color(const Vector3 &color)
{
	this->color = color;
}

const Vector3 &Light::get_color() const
{
	return color;
}

Vector3 Light::get_color(const Vector3 &pt) const
{
	float d = (get_position() - pt).length();
	return color * calc_attenuation(d) * intens;
}

void Light::set_attenuation(const Vector3 &atten)
{
	this->atten = atten;
}

const Vector3 &Light::get_attenuation() const
{
	return atten;
}

float Light::calc_attenuation(float d) const
{
	float denom = atten.x + atten.y * d + atten.z * d * d;
	float at = denom == 0.0 ? 1.0 : 1.0 / denom;
	return at > 1.0 ? 1.0 : at;
}

void Light::draw(bool emph) const
{
}

bool Light::intersect(const Ray &ray, RayHit *hit) const
{
	return false;	// TODO
}
