#ifndef MIN3D_H_
#define MIN3D_H_

#include "inttypes.h"

/* state toggles */
enum {
	M3D_DEPTH_TEST,
	M3D_CULL_FACE,
	M3D_LIGHTING,
	M3D_LIGHT0,
	M3D_LIGHT1,
	M3D_LIGHT2,
	M3D_LIGHT3
};

/* buffer bits */
enum {
	M3D_COLOR_BUFFER_BIT	= 1,
	M3D_DEPTH_BUFFER_BIT	= 2
};

/* primitives */
enum {
	M3D_POINTS = 1,
	M3D_LINES = 2,
	M3D_TRIANGLES = 3,
	M3D_QUADS = 4
};

/* matrix mode */
enum {
	M3D_MODELVIEW,
	M3D_PROJECTION
};

struct m3d_image {
	int xsz, ysz;
	uint32_t *pixels;
};

#ifdef __cplusplus
extern "C" {
#endif

int m3d_init(void);
void m3d_shutdown(void);

void m3d_set_buffers(struct m3d_image *cbuf, uint16_t *zbuf);
void m3d_clear_color(float r, float g, float b);
void m3d_clear(unsigned int bmask);

void m3d_enable(int bit);
void m3d_disable(int bit);

void m3d_viewport(int x, int y, int xsz, int ysz);

/* matrix stack */
void m3d_matrix_mode(int mode);
void m3d_push_matrix(void);
void m3d_pop_matrix(void);
void m3d_load_identity(void);
void m3d_load_matrix(const float *m);
void m3d_load_transpose_matrix(const float *m);
void m3d_mult_matrix(const float *m);
void m3d_mult_transpose_matrix(const float *m);
void m3d_translate(float x, float y, float z);
void m3d_rotate(float angle, float x, float y, float z);
void m3d_scale(float x, float y, float z);
void m3d_frustum(float left, float right, float bottom, float top, float nr, float fr);
void m3d_perspective(float vfov, float aspect, float znear, float zfar);

/* drawing */
void m3d_vertex_array(const float *varr);
void m3d_normal_array(const float *narr);
void m3d_color_array(const float *carr);
void m3d_texcoord_array(const float *tcarr);

void m3d_draw(int prim, int vcount);
void m3d_draw_indexed(int prim, const unsigned int *idxarr, int icount);

/* immediate mode interface */
void m3d_begin(int prim);
void m3d_end(void);
void m3d_vertex(float x, float y, float z);
void m3d_normal(float x, float y, float z);
void m3d_color(float x, float y, float z);
void m3d_texcoord(float x, float y);

#ifdef __cplusplus
}
#endif

#endif	/* MIN3D_H_ */
