#ifndef SCENE_H_
#define SCENE_H_

#include <stdio.h>
#include <string>
#include "vector.h"
#include "snode.h"
#include "object.h"
#include "light.h"
#include "camera.h"
#include "raytrace.h"

enum { SCN_BG_LOW, SCN_BG_MID, SCN_BG_HIGH };

class Scene {
private:
	char *name;
	vector<SceneNode*> nodes;

	vector<Object*> objects;
	vector<Light*> lights;
	vector<Camera*> cameras;
	Camera *active_cam;

	Vector3 bg[3];
	Vector3 ambient;
	float fog_exp;

	vector<int> sel;

public:
	Scene();
	~Scene();

	void clear();

	void set_name(const char *name);
	const char *get_name() const;

	void set_background(const Vector3 &col, int idx = -1);
	const Vector3 &get_background(int idx = -1) const;
	Vector3 get_background(const Ray &ray) const;

	void set_ambient(const Vector3 &col);
	const Vector3 &get_ambient() const;

	void add(SceneNode *node);

	int get_node_count() const;
	int get_object_count() const;
	int get_light_count() const;
	int get_camera_count() const;

	SceneNode *get_node(int idx);
	const SceneNode *get_node(int idx) const;
	Object *get_object(int idx);
	const Object *get_object(int idx) const;
	Light *get_light(int idx);
	const Light *get_light(int idx) const;
	Camera *get_camera(int idx);
	const Camera *get_camera(int idx) const;

	void set_active_camera(Camera *cam);
	Camera *get_active_camera() const;

	void draw() const;

	void select(int s);
	void clear_selection();
	int get_selection_count() const;
	int get_selection(int idx = 0) const;

	bool intersect(const Ray &ray, RayHit *hit = 0) const;

	bool load(const char *fname);
	bool save(const char *fname) const;

	bool load(FILE *fp);
	bool save(FILE *fp) const;
};

#endif	// SCENE_H_
