/* vi:set ft=cpp: */
#ifndef VECTOR_H_
#define VECTOR_H_

#include <stdlib.h>

template <class T>
class vector {
private:
	T *data;
	size_t num_items, max_items;

public:
	vector()
	{
		data = 0;
		num_items = max_items = 0;
	}

	vector(const vector &v)
	{
		data = 0;
		num_items = max_items = 0;
		resize(v.size());

		for(size_t i=0; i<v.size(); i++) {
			data[i] = v.data[i];
		}
	}

	vector &operator =(const vector &v)
	{
		if(&v != this) {
			clear();
			resize(v.size());

			for(size_t i=0; i<v.size(); i++) {
				data[i] = v.data[i];
			}
		}
		return *this;
	}

	~vector()
	{
		clear();
	}

	void clear()
	{
		delete [] data;
		data = 0;
		max_items = num_items = 0;
	}

	bool empty() const
	{
		return num_items == 0;
	}

	size_t size() const
	{
		return num_items;
	}

	void resize(size_t nsz)
	{
		if(nsz > 0) {
			T *newdata = new T[nsz];
			if(!newdata) return;

			size_t num = num_items < nsz ? num_items : nsz;
			for(size_t i=0; i<num; i++) {
				newdata[i] = data[i];
			}

			delete [] data;
			data = newdata;
			max_items = nsz;
			num_items = num;
		} else {
			delete [] data;
			max_items = num_items = 0;
		}
	}

	void push_back(const T &item)
	{
		if(num_items >= max_items) {
			resize(max_items > 0 ? max_items * 2 : 8);
		}
		data[num_items++] = item;
	}

	void pop_back()
	{
		if(--num_items <= 0) {
			num_items = 0;
		}

		if(num_items < max_items / 3) {
			resize(max_items / 2);
		}
	}

	T &back()
	{
		return data[num_items - 1];
	}

	const T &back() const
	{
		return data[num_items - 1];
	}

	T &operator [](int idx)
	{
		return data[idx];
	}

	const T &operator [](int idx) const
	{
		return data[idx];
	}
};

#endif	// VECTOR_H_
